#include <vector>
using namespace std;

#include "glpanel.h"

//--------------------------------------------------------------------
GLPanel::GLPanel(QWidget *parent)
 : QGLWidget(parent)
{
 m_model = NULL;
 m_dspId = 0;
 m_updatedDsp = false;
 m_left = -1.0;
 m_right = 11.0;
 m_bottom = -1.0;
 m_top = 11.0;
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
GLPanel::~GLPanel()
{
 if(m_dspId > 0)
  glDeleteLists(m_dspId, 1);
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::setModel(MyModel* _model)
{
 m_model = _model;
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void 
GLPanel::resetModelDisplay()
{
 if(m_model == NULL)
  return;
 m_updatedDsp = false;
 update();
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::initializeGL()
{
 // Set white as background color and clear window
 glClearColor(1.0, 1.0, 1.0, 1.0); 
 glClear(GL_COLOR_BUFFER_BIT);

 // enable smooth line display
 glEnable(GL_LINE_SMOOTH);
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::resizeGL(int _width, int _height)
{
 // Avoid division by zero
 if(_width == 0) _width = 1;

 // Store GL canvas sizes in object properties
 m_w = _width;
 m_h = _height;

 // Setup world space window limits based on model bounding box
 if( (m_model != NULL) && (!m_model->isEmpty()) )
 {
  m_model->getBoundBox(&m_left, &m_right, &m_bottom, &m_top);
  scaleWorldWindow(1.10);
 }
 else
 {
  scaleWorldWindow(1.0);
 }

 // Setup the viewport to canvas dimensions
 glViewport(0, 0, (GLint)m_w, (GLint)m_h);

 // Reset the coordinate system
 glMatrixMode(GL_PROJECTION); 
 glLoadIdentity();

 // Establish the clipping volume by setting up an
 // orthographic projection
 glOrtho(m_left, m_right, m_bottom, m_top, -1.0, 1.0);

 // Setup display in model coordinates
 glMatrixMode(GL_MODELVIEW); 
 glLoadIdentity();
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::paintGL()
{
 // Clear the buffer with the current clear color
 glClear(GL_COLOR_BUFFER_BIT);

 // Display model (if there is one)
 if( (m_model != NULL) && (!m_model->isEmpty()) )
 {
  if(!m_updatedDsp)
  {
   if(m_dspId > 0)
    glDeleteLists(m_dspId, 1);
   m_dspId = makeDisplayModel();
  }
  if(m_dspId > 0)
  {
   glCallList(m_dspId);
   m_updatedDsp = true;
  }
 }
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
GLuint 
GLPanel::makeDisplayModel()
{
 if( (m_model == NULL) || m_model->isEmpty() )
  return 0;

 GLuint list = glGenLists(1);
 glNewList(list, GL_COMPILE);

 // Display model polygon RGB color at its vertices
 // interpolating smoothly the color in the interior
 vector<Point> verts = m_model->getVerts();
 vector<Color> vcolors = m_model->getVColors();
 glShadeModel(GL_SMOOTH);
 glBegin(GL_TRIANGLES);
 for(int i = 0; i < verts.size(); i++)
 {
  glColor3d((GLdouble)vcolors[i].getR(),
            (GLdouble)vcolors[i].getG(),
            (GLdouble)vcolors[i].getB());
  glVertex2d((GLdouble)verts[i].getX(),
             (GLdouble)verts[i].getY());
 }
 glEnd();

 glEndList();
 return list;
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::scaleWorldWindow(double _scaleFac)
{
 double vpr;          // viewport distortion ratio
 double cx, cy;       // window center
 double sizex, sizey; // window sizes

 //Compute canvas viewport ratio.
 vpr = (double)m_h / (double)m_w;

 // Get current window center.
 /*** COMPLETE HERE - GLPANEL: 01 ***/


 /*** COMPLETE HERE - GLPANEL: 01 ***/

 // Set new window sizes based on scaling factor.
 /*** COMPLETE HERE - GLPANEL: 02 ***/


 /*** COMPLETE HERE - GLPANEL: 02 ***/

 // Adjust window to keep the same aspect ratio of the viewport.
 /*** COMPLETE HERE - GLPANEL: 03 ***/









 /*** COMPLETE HERE - GLPANEL: 03 ***/

 // Establish the clipping volume by setting up an
 // orthographic projection
 glMatrixMode(GL_PROJECTION); 
 glLoadIdentity();
 glOrtho(m_left, m_right, m_bottom, m_top, -1.0, 1.0);
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::panWorldWindow(double _panFacX, double _panFacY)
{
 // Compute pan distances in horizontal and vertical directions.
 double panX = (m_right - m_left)   * _panFacX;
 double panY = (m_top   - m_bottom) * _panFacY;

 // Shift current window.
 /*** COMPLETE HERE - GLPANEL: 04 ***/




 /*** COMPLETE HERE - GLPANEL: 04 ***/

 // Establish the clipping volume by setting up an
 // orthographic projection
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 glOrtho(m_left, m_right, m_bottom, m_top, -1.0, 1.0);
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::fitWorldToViewport()
{
 if(!m_model)
  return;
 m_model->getBoundBox(&m_left, &m_right, &m_bottom, &m_top);
 scaleWorldWindow(1.10);
 update();
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::zoomIn()
{
 if(!m_model)
  return;
 /*** COMPLETE HERE - GLPANEL: 05 ***/


 /*** COMPLETE HERE - GLPANEL: 05 ***/
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::zoomOut()
{
 if(!m_model)
  return;
 /*** COMPLETE HERE - GLPANEL: 06 ***/


 /*** COMPLETE HERE - GLPANEL: 06 ***/
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::panLeft()
{
 if(!m_model)
  return;
 /*** COMPLETE HERE - GLPANEL: 07 ***/


 /*** COMPLETE HERE - GLPANEL: 07 ***/
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::panRight()
{
 if(!m_model)
  return;
 /*** COMPLETE HERE - GLPANEL: 08 ***/


 /*** COMPLETE HERE - GLPANEL: 08 ***/
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::panDown()
{
 if(!m_model)
  return;
 /*** COMPLETE HERE - GLPANEL: 09 ***/


 /*** COMPLETE HERE - GLPANEL: 09 ***/
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
GLPanel::panUp()
{
 if(!m_model)
  return;
 /*** COMPLETE HERE - GLPANEL: 10 ***/


 /*** COMPLETE HERE - GLPANEL: 10 ***/
}
//--------------------------------------------------------------------
