#ifndef GLPANEL_H
#define GLPANEL_H

#include <QtOpenGL/QGLWidget>

#include "mymodel.h"

class GLPanel : public QGLWidget
{
 Q_OBJECT // must include this if you use Qt Signals & Slots

 public:
  GLPanel(QWidget *parent = 0);
  ~GLPanel();

  void     setModel(MyModel* _model);
  MyModel* getModel() { return m_model; };
  void     resetModelDisplay();

 protected:
  // Canvas predefined slots
  void     initializeGL();
  void     resizeGL(int _width, int _height);
  void     paintGL();

  // Display functions
  GLuint makeDisplayModel();

  // Functions to manage visualization window limits
  void scaleWorldWindow(double _scaleFac);
  void panWorldWindow(double _panFacX, double _panFacY);

 public:
  void fitWorldToViewport();
  void zoomIn();
  void zoomOut();
  void panLeft();
  void panRight();
  void panDown();
  void panUp();

 protected:
  MyModel* m_model;   // model to be displayed
  GLuint m_dspId;     // GL list index for model display
  bool m_updatedDsp;  // if true, model display is updated

  int m_w;            // width: GL canvas horizontal size
  int m_h;            // height: GL canvas vertical size
  GLdouble m_left;    // left limit of object space window
  GLdouble m_right;   // right limit of object space window
  GLdouble m_bottom;  // bottom limit of object space window
  GLdouble m_top;     // top limit of object space window
};

#endif // GLPANEL_H
